# $Name:  $ 
# $Author: dgp $ 
# $Date: 1997/03/24 23:07:31 $ 
#
# A set of Tcl procs for passing version information in CVS tags into
# the Tcl package mechanism.
#
# Assumes CVS properly configured to generate Name tags according to Tcl 
# version number conventions.  That problem solved separately.

# Check both existence of package command, and Tcl interpreter 7.4 or later.
# Either not true will raise error here.
package require Tcl 7.4

# proc PackageProvide CVSName packagePrefix
#	Takes the CVS "Name" keyword and extracts from it the package
#	name, version, and patchlevel information.  
#
#	Then this proc takes care of several package houskeeping tasks:
#
#	o Calls package provide
#	o Creates standard interface procs (for example, for the package
#	  Package with prefix pkg, the following procs are created)
#		OMFPACKAGE_PackagePrefix
#		OMFPKG_PackagePrefix
#		OMFPKG_PackageName
#		OMFPKG_Version
#		OMFPKG_PatchLevel
#		OMFPKG_FileNameBase
#	o Defines the standard globals if they're not already defined
#		omfpkg_library
#		omfpkg_debug
#
proc OMFCVS_PackageProvide {CVSName packagePrefix} {
  # The dots (.) at the beginning and end of the following regular expression
  # match the leading and trailing dollar signs ($) of a CVS keyword.
  # The $ is not explicitly matched because of the quoting hell involved.
  # ($ has meaning both in Tcl and in regular expressions)
  if {![regexp -nocase \
         {^.name: ([a-z]([a-z0-9_]*[a-z])?)-([0-9_]+)([abp])([0-9]+).*.$} \
         $CVSName match packageName ignore version level serial]} {
    error "CVS tag \"$CVSName\" can't be parsed into Tcl package information"
  }

  # Enforce Tcl package version number convention.
  # Decimal numbers separated by .'s (eg. 7.4)
  #	-- Too bad CVS doesn't allow '.' in its tags!
  regsub -all {_} $version {.} version

  # Publish existence of package to the package database
  package provide $packageName $version

  # Construct Tcl package patch level identifier (eg. 7.6p2)
  set patchLevel $version$level$serial

  # If this package includes C code, and that C code has chosen to define
  # the standard procs and globals itself rather than by a call to this 
  # routine, and this package has a Tcl part which does call this routine 
  # (that's what got us here), then we should not redefine the standard
  # procs and globals, but trust the ones established by the C code.
  # In that case the only reason this proc was called at all was to
  # make the call to package provide above.
  #
  # We do need to check, though, that the patch levels of the C part and
  # the Tcl part are in sync.  They might not be if the C code incorrectly
  # sourced the wrong initialization script.
  #
  # There are better solutions.
  # 1. If Tcl_PackageProvide is called in the C code, then any mismatch
  #    will be caught by the 'package provide' above.  However, that option
  #    is not available in Tcl 7.4, which we wish to serve.
  # 2. Rather than define the standard procs and globals itself, and as
  #    a replacement for a Tcl_PackageProvide call, the C code could call
  #    TclEval on a string containing a call to this function.  The only
  #    qualification is that the CvsTools package must already be loaded.
  #    It is loaded as part of Oommf_Init, so any C code which makes a
  #    call to Oommf_Init, can take this second approach anywhere following
  #    the successful return of Oommf_Init.
  #
  # The only circumstance I can imagine where neither of these options is
  # available is within the C code portion of the Oommf package itself.
  # That package is the only reason this check, and this lengthy explanation
  # are necessary.  Don't use them :) !
  #
  set PACKPREF [string toupper $packagePrefix]
  if {[string compare [info commands OMF${PACKPREF}_PatchLevel] ""]} {
    if {[string compare [OMF${PACKPREF}_PatchLevel] $patchLevel] != 0} {

      # Report the mismatch as an error
      append msg "\n$packageName version [OMF${PACKPREF}_PatchLevel] "
      append msg "loaded initialization script\n\n"
      append msg "\t[info script]\n\n"
      append msg "for version $patchLevel .  Versions must match.\n"
      error $msg
    }
    # When the patch levels do match, we do nothing else.  
    # Assume the binary has set the standard globals
#    return
  }

  # A Tcl package's library directory is the parent directory of the
  # directory which contains the files and index defining the package.
  # Presumably, if this file is being sourced then auto_path has a
  # suitable value.  However that value could be modifed, so for the
  # record keep the library directory for each package in a global
  # variable for future reference.
  #
  # (Should we also set a trace so that this becomes a read-only variable?
  #  Now it is documenting where the library came from rather than 
  #  directing where to go get it.)
  set library [file dirname [file dirname [info script]]]

  # Define standard global *_library
  global omf${packagePrefix}_library
  set omf${packagePrefix}_library $library

  # Define standard procs
  proc OMF${PACKPREF}_Version {} [list return $version]
  proc OMF${PACKPREF}_PatchLevel {} [list return $patchLevel]
  proc OMF${PACKPREF}_PackageName {} [list return $packageName]
  proc OMF${PACKPREF}_PackagePrefix {} [list return $packagePrefix]

  # Follow the same choice for our file name base as the Oommf project.
  if {![string compare [OMF_FileNameBase] omf]} {
    proc OMF${PACKPREF}_FileNameBase {} [list OMF${PACKPREF}_PackagePrefix]
  } else {
    proc OMF${PACKPREF}_FileNameBase {} \
      "string tolower \[OMF${PACKPREF}_PackageName\]"
  }

  # Define the one unconventional proc to help discover the package prefix
  # CAREFUL -- only do this if it doesn't already exist! i
  # (Think package name == package prefix !)
  if {![string compare \
      [info commands OMF[string toupper $packageName]_PackagePrefix] \
      ""]} {
    proc OMF[string toupper $packageName]_PackagePrefix {} \
      [list OMF${PACKPREF}_PackagePrefix]
  }

  # Disable this package's debugging messages by default
  global omf${packagePrefix}_debug
  if {![info exists omf${packagePrefix}_debug]} {
    set omf${packagePrefix}_debug 0
  }

}

# Add CvsTools package to database; define standard global variables
OMFCVS_PackageProvide {^Name: CvsTools-0_0a0^} cvs

